@@ -7,7 +7,7 @@ RDF::Query - An RDF query implementation of SPARQL/RDQL in Perl for use with RDF
 
 =head1 VERSION
 
-This document describes RDF::Query version 2.902_01.
+This document describes RDF::Query version 2.903_01.
 
 =head1 SYNOPSIS
 
@@ -33,9 +33,9 @@ information on RDQL.
 =head1 CHANGES IN VERSION 2.900
 
 The 2.9xx versions of RDF::Query introduce some significant changes that will
-lead to a stable 3.000 release supporting SPARQL 1.1. Version 2.902_01 introduces
+lead to a stable 3.000 release supporting SPARQL 1.1. Version 2.902 introduces
 the SPARQL 1.1 features up to date with the SPARQL 1.1 working drafts as of its
-release date. Version 2.902_01 also is the first version to require use of
+release date. Version 2.902 also is the first version to require use of
 RDF::Trine for the underlying RDF store. This change means that RDF::Core is
 no longer supported, and while Redland is still supported, its handling of
 "contexts" (named graphs) means that existing RDF triples stored in Redland
@@ -115,7 +115,7 @@ use Log::Log4perl qw(:easy);
 Log::Log4perl->easy_init($ERROR);
 
 no warnings 'numeric';
-use RDF::Trine 0.124;
+use RDF::Trine 0.126;
 require RDF::Query::Functions;	# (needs to happen at runtime because some of the functions rely on RDF::Query being fully loaded (to call add_hook(), for example))
 								# all the built-in functions including:
 								#     datatype casting, language ops, logical ops,
@@ -137,7 +137,7 @@ use RDF::Query::Plan;
 
 our ($VERSION, $DEFAULT_PARSER);
 BEGIN {
-	$VERSION		= '2.902_01';
+	$VERSION		= '2.903_01';
 	$DEFAULT_PARSER	= 'sparql11';
 }
 
@@ -181,8 +181,8 @@ sub new {
 	my ($baseuri, $languri, $lang, %options);
 	if (@_ and ref($_[0])) {
 		%options	= %{ shift() };
-		$lang		= $options{ lang };
-		$baseuri	= $options{ base };
+		$lang		= delete $options{ lang };
+		$baseuri	= delete $options{ base };
 	} else {
 		($baseuri, $languri, $lang, %options)	= @_;
 	}
@@ -205,7 +205,7 @@ sub new {
 		$baseuri	= RDF::Query::Node::Resource->new( $baseuri );
 	}
 	
-	my $update	= ($options{update} ? 1 : 0);
+	my $update	= ((delete $options{update}) ? 1 : 0);
 	my $pclass	= $names{ $lang } || $uris{ $languri } || $names{ $DEFAULT_PARSER };
 	my $parser	= $pclass->new();
 	my $parsed	= $parser->parse( $query, $baseuri, $update );
@@ -215,8 +215,8 @@ sub new {
 					parser			=> $parser,
 					parsed			=> $parsed,
 				);
-	if ($options{load_data}) {
-		$self->{load_data}	= $options{load_data};
+	if (exists $options{load_data}) {
+		$self->{load_data}	= delete $options{load_data};
 	} elsif ($pclass =~ /^RDF::Query::Parser::(RDQL|SPARQL)$/) {
 		$self->{load_data}	= 1;
 	} else {
@@ -229,31 +229,36 @@ sub new {
 	}
 	
 	if (defined $options{defines}) {
-		@{ $self->{options} }{ keys %{ $options{defines} } }	= values %{ $options{defines} };
+		@{ $self->{options} }{ keys %{ $options{defines} } }	= values %{ delete $options{defines} };
 	}
 	
 	if ($options{logger}) {
 		$l->debug("got external logger");
-		$self->{logger}	= $options{logger};
+		$self->{logger}	= delete $options{logger};
 	}
 	
-	if (my $opt = $options{optimize}) {
+	if (my $opt = delete $options{optimize}) {
 		$l->debug("got optimization flag: $opt");
 		$self->{optimize}	= $opt;
 	} else {
 		$self->{optimize}	= 0;
 	}
 	
-	if (my $opt = $options{force_no_optimization}) {
+	if (my $opt = delete $options{force_no_optimization}) {
 		$l->debug("got force_no_optimization flag");
 		$self->{force_no_optimization}	= 1;
 	}
 	
-	if (my $time = $options{optimistic_threshold_time}) {
+	if (my $time = delete $options{optimistic_threshold_time}) {
 		$l->debug("got optimistic_threshold_time flag");
 		$self->{optimistic_threshold_time}	= $time;
 	}
 	
+	my @leftover	= keys %options;
+	if (@leftover) {
+		warn "Unrecognized options passed to $class->new:\n\t" . join("\n\t", sort @leftover);
+	}
+	
 	# add rdf as a default namespace to RDQL queries
 	if ($pclass eq 'RDF::Query::Parser::RDQL') {
 		$self->{parsed}{namespaces}{rdf}	= 'http://www.w3.org/1999/02/22-rdf-syntax-ns#';
@@ -338,7 +343,8 @@ sub prepare {
 	$self->{model}		= $model;
 	
 	$l->trace("getting QEP...");
-	my $plan		= $self->query_plan( $context );
+	my %plan_args	= %{ $args{ planner_args } || {} };
+	my $plan		= $self->query_plan( $context, %plan_args );
 	$l->trace("-> done.");
 	
 	unless ($plan) {
@@ -475,6 +481,7 @@ current query.
 sub query_plan {
 	my $self	= shift;
 	my $context	= shift;
+	my %args	= @_;
 	my $parsed	= $self->{parsed};
 	my %constant_plan;
 	if (my $b = $self->{parsed}{bindings}) {
@@ -500,7 +507,7 @@ sub query_plan {
 	
 	my $algebra		= $self->pattern;
 	my $pclass		= $self->plan_class;
-	my @plans		= $pclass->generate_plans( $algebra, $context, %constant_plan );
+	my @plans		= $pclass->generate_plans( $algebra, $context, %args, %constant_plan );
 	
 	my $l		= Log::Log4perl->get_logger("rdf.query.plan");
 	if (wantarray) {
@@ -563,8 +570,7 @@ sub describe {
 	$self->{'describe_nodes'}	= [];
 	foreach my $node (@nodes) {
 		push(@{ $self->{'describe_nodes'} }, $node);
-		push(@streams, $model->get_statements( $node ));
-		push(@streams, $model->get_statements( undef, undef, $node ));
+		push(@streams, $model->bounded_description( $node ));
 	}
 	
 	my $ret	= sub {
@@ -1384,9 +1390,9 @@ L<http://www.perlrdf.org/>
 
  Gregory Todd Williams <gwilliams@cpan.org>
 
-=head1 COPYRIGHT
+=head1 LICENSE
 
-Copyright (c) 2005-2009 Gregory Todd Williams. All rights reserved. This
+Copyright (c) 2005-2010 Gregory Todd Williams. All rights reserved. This
 program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
