@@ -7,7 +7,7 @@ RDF::Query::Algebra::Construct - Algebra class for construct query results
 
 =head1 VERSION
 
-This document describes RDF::Query::Algebra::Construct version 2.902_01.
+This document describes RDF::Query::Algebra::Construct version 2.903_01.
 
 =cut
 
@@ -28,7 +28,7 @@ use RDF::Trine::Iterator qw(sgrep);
 
 our ($VERSION);
 BEGIN {
-	$VERSION	= '2.902_01';
+	$VERSION	= '2.903_01';
 }
 
 ######################################################################
@@ -172,16 +172,16 @@ sub referenced_variables {
 	return RDF::Query::_uniq($self->pattern->referenced_variables);
 }
 
-=item C<< binding_variables >>
+=item C<< potentially_bound >>
 
 Returns a list of the variable names used in this algebra expression that will
 bind values during execution.
 
 =cut
 
-sub binding_variables {
+sub potentially_bound {
 	my $self	= shift;
-	return RDF::Query::_uniq($self->pattern->binding_variables);
+	return $self->pattern->potentially_bound;
 }
 
 =item C<< definite_variables >>
