@@ -7,7 +7,7 @@ RDF::Query::Algebra::Extend - Algebra class for extending the variable projectio
 
 =head1 VERSION
 
-This document describes RDF::Query::Algebra::Extend version 2.902_01.
+This document describes RDF::Query::Algebra::Extend version 2.903_01.
 
 =cut
 
@@ -28,7 +28,7 @@ use RDF::Trine::Iterator qw(sgrep);
 
 our ($VERSION);
 BEGIN {
-	$VERSION	= '2.902_01';
+	$VERSION	= '2.903_01';
 }
 
 ######################################################################
@@ -139,7 +139,6 @@ sub _from_sse {
 			
 			if (m/^\s*[)]/) {
 				s/^\s*[)]\s*//;
-				warn "extend: " . Dumper(\@nodes);
 				return RDF::Query::Algebra::Extend->new( $pattern, \@nodes );
 			} else {
 				throw RDF::Trine::Error -text => "Cannot parse end-of-extend from SSE string: >>$_<<";
@@ -172,10 +171,13 @@ sub as_sparql {
 			push(@vars, $k);
 		}
 	}
-	my $pvars	= join(' ', map { '?' . $_ } sort $self->pattern->referenced_variables);
-	my $svars	= join(' ', sort @vars);
-	my $vars	= ($pvars eq $svars) ? '*' : join(' ', @vars);
-	return join(' ', $vars, 'WHERE', $self->pattern->as_sparql( $context, $indent ));
+	
+	my $ggp		= $self->pattern->as_sparql( $context, $indent );
+	my $sparql	= $ggp;
+	foreach my $v (@vars) {
+		$sparql	.=	"\n${indent}BIND" . $v;
+	}
+	return $sparql;
 }
 
 =item C<< as_hash >>
@@ -223,21 +225,22 @@ sub referenced_variables {
 	return RDF::Query::_uniq(@vars);
 }
 
-=item C<< binding_variables >>
+=item C<< potentially_bound >>
 
 Returns a list of the variable names used in this algebra expression that will
 bind values during execution.
 
 =cut
 
-sub binding_variables {
+sub potentially_bound {
 	my $self	= shift;
-	my @vars	= $self->pattern->binding_variables;
+	my @vars;
+	push(@vars, $self->pattern->potentially_bound);
 	foreach my $v (@{ $self->vars }) {
 		if ($v->isa('RDF::Query::Node::Variable')) {
 			push(@vars, $v->name);
 		} else {
-			push(@vars, $v->binding_variables);
+			push(@vars, $v->potentially_bound);
 		}
 	}
 	return RDF::Query::_uniq(@vars);
