@@ -7,7 +7,7 @@ RDF::Query::Algebra::Filter - Algebra class for Filter expressions
 
 =head1 VERSION
 
-This document describes RDF::Query::Algebra::Filter version 2.902_01.
+This document describes RDF::Query::Algebra::Filter version 2.903_01.
 
 =cut
 
@@ -29,7 +29,7 @@ use RDF::Trine::Iterator qw(sgrep smap swatch);
 
 our ($VERSION);
 BEGIN {
-	$VERSION	= '2.902_01';
+	$VERSION	= '2.903_01';
 }
 
 ######################################################################
@@ -187,16 +187,16 @@ sub referenced_variables {
 	}
 }
 
-=item C<< binding_variables >>
+=item C<< potentially_bound >>
 
 Returns a list of the variable names used in this algebra expression that will
 bind values during execution.
 
 =cut
 
-sub binding_variables {
+sub potentially_bound {
 	my $self	= shift;
-	return $self->pattern->binding_variables;
+	return $self->pattern->potentially_bound;
 }
 
 =item C<< definite_variables >>
