@@ -7,7 +7,7 @@ RDF::Query::Expression::Binary - Algebra class for binary expressions
 
 =head1 VERSION
 
-This document describes RDF::Query::Expression::Binary version 2.902_01.
+This document describes RDF::Query::Expression::Binary version 2.903_01.
 
 =cut
 
@@ -27,7 +27,7 @@ use Carp qw(carp croak confess);
 
 our ($VERSION);
 BEGIN {
-	$VERSION	= '2.902_01';
+	$VERSION	= '2.903_01';
 }
 
 ######################################################################
@@ -97,18 +97,42 @@ sub evaluate {
 	$l->debug("Binary Operator '$op': " . Dumper($lhs, $rhs));
 	
 	if ($op =~ m#^[-+/*]$#) {
-		my $type	= $self->promote_type( $op, $lhs, $rhs );
+		my $type	= $self->promote_type( $op, $lhs->literal_datatype, $rhs->literal_datatype );
 		my $value;
 		if ($op eq '+') {
-			$value	= $lhs->numeric_value + $rhs->numeric_value;
+			my $lhsv	= $lhs->numeric_value;
+			my $rhsv	= $rhs->numeric_value;
+			if (defined($lhsv) and defined($rhsv)) {
+				$value		= $lhsv + $rhsv;
+			} else {
+				throw RDF::Query::Error::ComparisonError -text => "Cannot evaluate infix:<+> on non-numeric types";
+			}
 		} elsif ($op eq '-') {
-			$value	= $lhs->numeric_value - $rhs->numeric_value;
+			my $lhsv	= $lhs->numeric_value;
+			my $rhsv	= $rhs->numeric_value;
+			if (defined($lhsv) and defined($rhsv)) {
+				$value		= $lhsv - $rhsv;
+			} else {
+				throw RDF::Query::Error::ComparisonError -text => "Cannot evaluate infix:<-> on non-numeric types";
+			}
 		} elsif ($op eq '*') {
-			$value	= $lhs->numeric_value * $rhs->numeric_value;
+			my $lhsv	= $lhs->numeric_value;
+			my $rhsv	= $rhs->numeric_value;
+			if (defined($lhsv) and defined($rhsv)) {
+				$value		= $lhsv * $rhsv;
+			} else {
+				throw RDF::Query::Error::ComparisonError -text => "Cannot evaluate infix:<*> on non-numeric types";
+			}
 		} elsif ($op eq '/') {
-			$value	= $lhs->numeric_value / $rhs->numeric_value;
+			my $lhsv	= $lhs->numeric_value;
+			my $rhsv	= $rhs->numeric_value;
+			if (defined($lhsv) and defined($rhsv)) {
+				$value		= $lhsv / $rhsv;
+			} else {
+				throw RDF::Query::Error::ComparisonError -text => "Cannot evaluate infix:</> on non-numeric types";
+			}
 		} else {
-			die;
+			throw RDF::Query::Error::ExecutionError -text => "Unrecognized binary operator '$op'";
 		}
 		return RDF::Query::Node::Literal->new( $value, undef, $type );
 	} elsif ($op =~ m#^([<>]=?)|!?=$#) {
@@ -140,7 +164,7 @@ sub evaluate {
 		} elsif ($op eq '!=') {
 			$bool	= ($lhs != $rhs);
 		} else {
-			die;
+			throw RDF::Query::Error::ExecutionError -text => "Unrecognized binary operator '$op'";
 		}
 		
 		my $value	= ($bool) ? 'true' : 'false';
@@ -172,10 +196,10 @@ my %rel	= (
 	"${xsd}double"				=> 15,
 );
 
-=item C<< promote_type ( $op, $lhs, $rhs ) >>
+=item C<< promote_type ( $op, $lhs_datatype, $rhs_datatype ) >>
 
 Returns the XSD type URI (as a string) for the resulting value of performing the
-supplied operation on the arguments.
+supplied operation on arguments of the indicated XSD types.
 
 =cut
 
@@ -183,7 +207,7 @@ sub promote_type {
 	my $self	= shift;
 	my $op		= shift;
 	no warnings 'uninitialized';
-	my @types	= sort { $rel{$b} <=> $rel{$a} } map { $_->literal_datatype } @_;
+	my @types	= sort { $rel{$b} <=> $rel{$a} } @_;
 	
 	my $type	= $types[0];
 	$type		= "${xsd}integer" if ($integer_types{ $type });
