@@ -7,7 +7,7 @@ RDF::Query::Plan::Extend - Executable query plan for Extends.
 
 =head1 VERSION
 
-This document describes RDF::Query::Plan::Extend version 2.902_01.
+This document describes RDF::Query::Plan::Extend version 2.903_01.
 
 =head1 METHODS
 
@@ -27,7 +27,7 @@ use Scalar::Util qw(blessed);
 
 our ($VERSION);
 BEGIN {
-	$VERSION	= '2.902_01';
+	$VERSION	= '2.903_01';
 }
 
 ######################################################################
@@ -58,8 +58,10 @@ sub new {
 sub execute ($) {
 	my $self	= shift;
 	my $context	= shift;
+	my $l		= Log::Log4perl->get_logger("rdf.query.plan.extend");
+	$l->trace( "executing extend plan: " . $self->sse );
 	if ($self->state == $self->OPEN) {
-		throw RDF::Query::Error::ExecutionError -text => "PROJECT plan can't be executed while already open";
+		throw RDF::Query::Error::ExecutionError -text => "EXTEND plan can't be executed while already open";
 	}
 	my $plan	= $self->[1];
 	$plan->execute( $context );
@@ -103,7 +105,7 @@ sub next {
 		my $query	= $self->[0]{context}->query;
 		
 		local($query->{_query_row_cache})	= {};
-		my $proj	= $row->project( @{ $keys } );
+#		my $proj	= $row->project( @{ $keys } );
 		my $ok	= 1;
 		try {
 			foreach my $e (@$exprs) {
@@ -118,10 +120,13 @@ sub next {
 				}
 				$row->{ $name }	= $value;
 			}
+		} catch RDF::Query::Error with {
+			$l->trace( "- evaluating extend expression resulted in an error; dropping the variable binding" );
 		} otherwise {
 			$ok	= 0;
 		};
 		next unless ($ok);
+		$l->trace( "Extended result: $row" );
 		return $row;
 	}
 }
