@@ -7,7 +7,7 @@ RDF::Query::Plan::Filter - Executable query plan for Filters.
 
 =head1 VERSION
 
-This document describes RDF::Query::Plan::Filter version 2.902_01.
+This document describes RDF::Query::Plan::Filter version 2.903_01.
 
 =head1 METHODS
 
@@ -26,7 +26,7 @@ use RDF::Query::Error qw(:try);
 
 our ($VERSION);
 BEGIN {
-	$VERSION	= '2.902_01';
+	$VERSION	= '2.903_01';
 }
 
 ######################################################################
@@ -111,7 +111,7 @@ sub next {
 	my $l		= Log::Log4perl->get_logger("rdf.query.plan.filter");
 	while (1) {
 		my $row	= $plan->next;
-		unless ($row) {
+		unless (defined($row)) {
 			$l->debug("no remaining rows in filter");
 			return;
 		}
