@@ -7,7 +7,7 @@ RDF::Query::Plan::Join::PushDownNestedLoop - Executable query plan for nested lo
 
 =head1 VERSION
 
-This document describes RDF::Query::Plan::Join::PushDownNestedLoop version 2.902_01.
+This document describes RDF::Query::Plan::Join::PushDownNestedLoop version 2.903_01.
 
 =head1 METHODS
 
@@ -27,7 +27,7 @@ use Data::Dumper;
 
 our ($VERSION);
 BEGIN {
-	$VERSION	= '2.902_01';
+	$VERSION	= '2.903_01';
 	$RDF::Query::Plan::Join::JOIN_CLASSES{ 'RDF::Query::Plan::Join::PushDownNestedLoop' }++;
 }
 
@@ -51,6 +51,11 @@ sub new {
 		# for example.
 		throw RDF::Query::Error::MethodInvocationError -text => "PushDownNestedLoop join does not support optional patterns as RHS due to bottom-up variable scoping rules (use NestedLoop instead)";
 	}
+	
+	if ($rhs->sse =~ /aggregate/sm) {
+		throw RDF::Query::Error::MethodInvocationError -text => "PushDownNestedLoop join does not support aggregates in the RHS due to aggregate group fragmentation";
+	}
+	
 	my $self	= $class->SUPER::new( $lhs, $rhs, $opt );
 	return $self;
 }
@@ -167,7 +172,7 @@ sub close {
 	delete $self->[0]{outer};
 	delete $self->[0]{needs_new_outer};
 	delete $self->[0]{inner_count};
-	if ($self->lhs->state == $self->lhs->OPEN) {
+	if (blessed($self->lhs) and $self->lhs->state == $self->lhs->OPEN) {
 		$self->lhs->close();
 	}
 	$self->SUPER::close();
